# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .ExperienceModel import ExperienceModel
from .options.ExperienceModelOptions import ExperienceModelOptions
from .Threshold import Threshold
from .options.ThresholdOptions import ThresholdOptions
from .AcquireActionRate import AcquireActionRate
from .options.AcquireActionRateOptions import AcquireActionRateOptions
from .enum.AcquireActionRateMode import AcquireActionRateMode
from .options.AcquireActionRateModeIsDoubleOptions import AcquireActionRateModeIsDoubleOptions
from .options.AcquireActionRateModeIsBigOptions import AcquireActionRateModeIsBigOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from ...core.model import ScriptSetting
from ...core.model import LogSetting

from ..ref.NamespaceRef import NamespaceRef
from .CurrentMasterData import CurrentMasterData
from .EntryModel import EntryModel

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    description: Optional[str] = None
    entry_script: Optional[ScriptSetting] = None
    duplicate_entry_script: Optional[ScriptSetting] = None
    log_setting: Optional[LogSetting] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "Dictionary_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.description = options.description if options.description else None
        self.entry_script = options.entry_script if options.entry_script else None
        self.duplicate_entry_script = options.duplicate_entry_script if options.duplicate_entry_script else None
        self.log_setting = options.log_setting if options.log_setting else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Dictionary::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.entry_script is not None:
            properties["EntryScript"] = self.entry_script.properties(
            )
        if self.duplicate_entry_script is not None:
            properties["DuplicateEntryScript"] = self.duplicate_entry_script.properties(
            )
        if self.log_setting is not None:
            properties["LogSetting"] = self.log_setting.properties(
            )

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )

    def master_data(
        self,
        entry_models: List[EntryModel],
    ) -> Namespace:
        CurrentMasterData(
            self.stack,
            self.name,
            entry_models,
        ).add_depends_on(
            self,
        )
        return self



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.ExecuteStampTaskLogOptions import ExecuteStampTaskLogOptions


class ExecuteStampTaskLog:
    timestamp: int
    task_id: str
    service: str
    method: str
    user_id: str
    action: str
    args: str

    def __init__(
        self,
        timestamp: int,
        task_id: str,
        service: str,
        method: str,
        user_id: str,
        action: str,
        args: str,
        options: Optional[ExecuteStampTaskLogOptions] = ExecuteStampTaskLogOptions(),
    ):
        self.timestamp = timestamp
        self.task_id = task_id
        self.service = service
        self.method = method
        self.user_id = user_id
        self.action = action
        self.args = args

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.timestamp is not None:
            properties["timestamp"] = self.timestamp
        if self.task_id is not None:
            properties["taskId"] = self.task_id
        if self.service is not None:
            properties["service"] = self.service
        if self.method is not None:
            properties["method"] = self.method
        if self.user_id is not None:
            properties["userId"] = self.user_id
        if self.action is not None:
            properties["action"] = self.action
        if self.args is not None:
            properties["args"] = self.args

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.SlotOptions import SlotOptions


class Slot:
    name: str
    property_id: str
    metadata: Optional[str] = None

    def __init__(
        self,
        name: str,
        property_id: str,
        options: Optional[SlotOptions] = SlotOptions(),
    ):
        self.name = name
        self.property_id = property_id
        self.metadata = options.metadata if options.metadata else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.property_id is not None:
            properties["propertyId"] = self.property_id
        if self.metadata is not None:
            properties["metadata"] = self.metadata

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join


class Statement:

    effect: str
    actions: List[str]

    @staticmethod
    def allow(
            actions: List[str],
    ) -> Statement:
        statement = Statement()
        statement.effect = "Allow"
        statement.actions = actions
        return statement

    @staticmethod
    def allow_all() -> Statement:
        statement = Statement()
        statement.effect = "Allow"
        statement.actions = ['*']
        return statement

    @staticmethod
    def deny(
            actions: List[str],
    ) -> Statement:
        statement = Statement()
        statement.effect = "Deny"
        statement.actions = actions
        return statement

    @staticmethod
    def deny_all() -> Statement:
        statement = Statement()
        statement.effect = "Deny"
        statement.actions = ['*']
        return statement

    def action(self, action: str) -> Statement:
        self.actions.append(action)
        return self

    def properties(self) -> Dict[str, Any]:
        return {
            "Effect": self.effect,
            "Actions": [
                action
                for action in self.actions
            ],
            "Resources": ['*']
        }



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from .Identifier import Identifier
from .AttachSecurityPolicy import AttachSecurityPolicy
from .SecurityPolicy import SecurityPolicy

from ..ref.UserRef import UserRef

from .options.UserOptions import UserOptions


class User(CdkResource):
    stack: Stack
    name: str
    description: Optional[str] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        options: Optional[UserOptions] = UserOptions(),
    ):
        super().__init__(
            "Identifier_User_" + name
        )

        self.stack = stack
        self.name = name
        self.description = options.description if options.description else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Identifier::User"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description

        return properties

    def ref(
        self,
    ) -> UserRef:
        return UserRef(
            self.name,
        )


    def attach(
        self,
        security_policy: SecurityPolicy,
    ) -> User:
        AttachSecurityPolicy(
            self.stack,
            self.name,
            security_policy.get_attr_security_policy_id(
            ).str(
            ),
        ).add_depends_on(
            self,
        ).add_depends_on(
            security_policy,
        )

        return self

    def attach_grn(
        self,
        security_policy_grn: str,
    ) -> User:
        AttachSecurityPolicy(
            self.stack,
            self.name,
            security_policy_grn,
        ).add_depends_on(
            self,
        )

        return self

    def identifier(
        self,
    ) -> Identifier:
        identifier =Identifier(
            self.stack,
            self.name,
        );
        identifier.add_depends_on(
            self,
        )

        return identifier

    def get_attr_user_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.UserId",
            None,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import ConsumeAction
from ...core.model import AcquireAction
from .SalesItem import SalesItem
from .SalesItemGroup import SalesItemGroup
from .options.DisplayItemOptions import DisplayItemOptions
from .options.DisplayItemTypeIsSalesItemOptions import DisplayItemTypeIsSalesItemOptions
from .options.DisplayItemTypeIsSalesItemGroupOptions import DisplayItemTypeIsSalesItemGroupOptions
from .enum.DisplayItemType import DisplayItemType


class DisplayItem:
    display_item_id: str
    type: DisplayItemType
    sales_item: Optional[SalesItem] = None
    sales_item_group: Optional[SalesItemGroup] = None
    sales_period_event_id: Optional[str] = None

    def __init__(
        self,
        display_item_id: str,
        type: DisplayItemType,
        options: Optional[DisplayItemOptions] = DisplayItemOptions(),
    ):
        self.display_item_id = display_item_id
        self.type = type
        self.sales_item = options.sales_item if options.sales_item else None
        self.sales_item_group = options.sales_item_group if options.sales_item_group else None
        self.sales_period_event_id = options.sales_period_event_id if options.sales_period_event_id else None

    @staticmethod
    def type_is_sales_item(
        display_item_id: str,
        sales_item: SalesItem,
        options: Optional[DisplayItemTypeIsSalesItemOptions] = DisplayItemTypeIsSalesItemOptions(),
    ) -> DisplayItem:
        return DisplayItem(
            display_item_id,
            DisplayItemType.SALES_ITEM,
            DisplayItemOptions(
                sales_item,
                options.sales_period_event_id,
            ),
        )

    @staticmethod
    def type_is_sales_item_group(
        display_item_id: str,
        sales_item_group: SalesItemGroup,
        options: Optional[DisplayItemTypeIsSalesItemGroupOptions] = DisplayItemTypeIsSalesItemGroupOptions(),
    ) -> DisplayItem:
        return DisplayItem(
            display_item_id,
            DisplayItemType.SALES_ITEM_GROUP,
            DisplayItemOptions(
                sales_item_group,
                options.sales_period_event_id,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.display_item_id is not None:
            properties["displayItemId"] = self.display_item_id
        if self.type is not None:
            properties["type"] = self.type.value
        if self.sales_item is not None:
            properties["salesItem"] = self.sales_item.properties(
            )
        if self.sales_item_group is not None:
            properties["salesItemGroup"] = self.sales_item_group.properties(
            )
        if self.sales_period_event_id is not None:
            properties["salesPeriodEventId"] = self.sales_period_event_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.AccessTokenOptions import AccessTokenOptions


class AccessToken:
    owner_id: str
    user_id: str
    expire: int
    time_offset: int

    def __init__(
        self,
        owner_id: str,
        user_id: str,
        expire: int,
        time_offset: int,
        options: Optional[AccessTokenOptions] = AccessTokenOptions(),
    ):
        self.owner_id = owner_id
        self.user_id = user_id
        self.expire = expire
        self.time_offset = time_offset

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.owner_id is not None:
            properties["ownerId"] = self.owner_id
        if self.user_id is not None:
            properties["userId"] = self.user_id
        if self.expire is not None:
            properties["expire"] = self.expire
        if self.time_offset is not None:
            properties["timeOffset"] = self.time_offset

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.VersionOptions import VersionOptions


class Version:
    major: int
    minor: int
    micro: int

    def __init__(
        self,
        major: int,
        minor: int,
        micro: int,
        options: Optional[VersionOptions] = VersionOptions(),
    ):
        self.major = major
        self.minor = minor
        self.micro = micro

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.major is not None:
            properties["major"] = self.major
        if self.minor is not None:
            properties["minor"] = self.minor
        if self.micro is not None:
            properties["micro"] = self.micro

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .RecoverStaminaByUserId import RecoverStaminaByUserId
from .RaiseMaxValueByUserId import RaiseMaxValueByUserId
from .SetMaxValueByUserId import SetMaxValueByUserId
from .SetRecoverIntervalByUserId import SetRecoverIntervalByUserId
from .SetRecoverValueByUserId import SetRecoverValueByUserId
from .DecreaseMaxValueByUserId import DecreaseMaxValueByUserId
from .ConsumeStaminaByUserId import ConsumeStaminaByUserId


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .Event import Event
from .options.EventOptions import EventOptions
from .enum.EventScheduleType import EventScheduleType
from .enum.EventRepeatType import EventRepeatType
from .enum.EventRepeatBeginDayOfWeek import EventRepeatBeginDayOfWeek
from .enum.EventRepeatEndDayOfWeek import EventRepeatEndDayOfWeek
from .options.EventScheduleTypeIsAbsoluteOptions import EventScheduleTypeIsAbsoluteOptions
from .options.EventScheduleTypeIsRelativeOptions import EventScheduleTypeIsRelativeOptions
from .options.EventRepeatTypeIsAlwaysOptions import EventRepeatTypeIsAlwaysOptions
from .options.EventRepeatTypeIsDailyOptions import EventRepeatTypeIsDailyOptions
from .options.EventRepeatTypeIsWeeklyOptions import EventRepeatTypeIsWeeklyOptions
from .options.EventRepeatTypeIsMonthlyOptions import EventRepeatTypeIsMonthlyOptions
from .RepeatSchedule import RepeatSchedule
from .options.RepeatScheduleOptions import RepeatScheduleOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from ...core.model import ScriptSetting
from ...core.model import NotificationSetting
from ...core.model import LogSetting

from ..ref.NamespaceRef import NamespaceRef
from .CurrentMasterData import CurrentMasterData
from .RatingModel import RatingModel
from .enum.NamespaceCreateGatheringTriggerType import NamespaceCreateGatheringTriggerType
from .enum.NamespaceCompleteMatchmakingTriggerType import NamespaceCompleteMatchmakingTriggerType

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    create_gathering_trigger_type: NamespaceCreateGatheringTriggerType
    complete_matchmaking_trigger_type: NamespaceCompleteMatchmakingTriggerType
    description: Optional[str] = None
    enable_rating: Optional[bool] = None
    create_gathering_trigger_realtime_namespace_id: Optional[str] = None
    create_gathering_trigger_script_id: Optional[str] = None
    complete_matchmaking_trigger_realtime_namespace_id: Optional[str] = None
    complete_matchmaking_trigger_script_id: Optional[str] = None
    change_rating_script: Optional[ScriptSetting] = None
    join_notification: Optional[NotificationSetting] = None
    leave_notification: Optional[NotificationSetting] = None
    complete_notification: Optional[NotificationSetting] = None
    change_rating_notification: Optional[NotificationSetting] = None
    log_setting: Optional[LogSetting] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        create_gathering_trigger_type: NamespaceCreateGatheringTriggerType,
        complete_matchmaking_trigger_type: NamespaceCompleteMatchmakingTriggerType,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "Matchmaking_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.create_gathering_trigger_type = create_gathering_trigger_type
        self.complete_matchmaking_trigger_type = complete_matchmaking_trigger_type
        self.description = options.description if options.description else None
        self.enable_rating = options.enable_rating if options.enable_rating else None
        self.create_gathering_trigger_realtime_namespace_id = options.create_gathering_trigger_realtime_namespace_id if options.create_gathering_trigger_realtime_namespace_id else None
        self.create_gathering_trigger_script_id = options.create_gathering_trigger_script_id if options.create_gathering_trigger_script_id else None
        self.complete_matchmaking_trigger_realtime_namespace_id = options.complete_matchmaking_trigger_realtime_namespace_id if options.complete_matchmaking_trigger_realtime_namespace_id else None
        self.complete_matchmaking_trigger_script_id = options.complete_matchmaking_trigger_script_id if options.complete_matchmaking_trigger_script_id else None
        self.change_rating_script = options.change_rating_script if options.change_rating_script else None
        self.join_notification = options.join_notification if options.join_notification else None
        self.leave_notification = options.leave_notification if options.leave_notification else None
        self.complete_notification = options.complete_notification if options.complete_notification else None
        self.change_rating_notification = options.change_rating_notification if options.change_rating_notification else None
        self.log_setting = options.log_setting if options.log_setting else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Matchmaking::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.enable_rating is not None:
            properties["EnableRating"] = self.enable_rating
        if self.create_gathering_trigger_type is not None:
            properties["CreateGatheringTriggerType"] = self.create_gathering_trigger_type
        if self.create_gathering_trigger_realtime_namespace_id is not None:
            properties["CreateGatheringTriggerRealtimeNamespaceId"] = self.create_gathering_trigger_realtime_namespace_id
        if self.create_gathering_trigger_script_id is not None:
            properties["CreateGatheringTriggerScriptId"] = self.create_gathering_trigger_script_id
        if self.complete_matchmaking_trigger_type is not None:
            properties["CompleteMatchmakingTriggerType"] = self.complete_matchmaking_trigger_type
        if self.complete_matchmaking_trigger_realtime_namespace_id is not None:
            properties["CompleteMatchmakingTriggerRealtimeNamespaceId"] = self.complete_matchmaking_trigger_realtime_namespace_id
        if self.complete_matchmaking_trigger_script_id is not None:
            properties["CompleteMatchmakingTriggerScriptId"] = self.complete_matchmaking_trigger_script_id
        if self.change_rating_script is not None:
            properties["ChangeRatingScript"] = self.change_rating_script.properties(
            )
        if self.join_notification is not None:
            properties["JoinNotification"] = self.join_notification.properties(
            )
        if self.leave_notification is not None:
            properties["LeaveNotification"] = self.leave_notification.properties(
            )
        if self.complete_notification is not None:
            properties["CompleteNotification"] = self.complete_notification.properties(
            )
        if self.change_rating_notification is not None:
            properties["ChangeRatingNotification"] = self.change_rating_notification.properties(
            )
        if self.log_setting is not None:
            properties["LogSetting"] = self.log_setting.properties(
            )

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )

    def master_data(
        self,
        rating_models: List[RatingModel],
    ) -> Namespace:
        CurrentMasterData(
            self.stack,
            self.name,
            rating_models,
        ).add_depends_on(
            self,
        )
        return self



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .MaxStaminaTable import MaxStaminaTable
from .RecoverIntervalTable import RecoverIntervalTable
from .RecoverValueTable import RecoverValueTable
from .options.StaminaModelOptions import StaminaModelOptions


class StaminaModel:
    name: str
    recover_interval_minutes: int
    recover_value: int
    initial_capacity: int
    is_overflow: bool
    metadata: Optional[str] = None
    max_capacity: Optional[int] = None
    max_stamina_table: Optional[MaxStaminaTable] = None
    recover_interval_table: Optional[RecoverIntervalTable] = None
    recover_value_table: Optional[RecoverValueTable] = None

    def __init__(
        self,
        name: str,
        recover_interval_minutes: int,
        recover_value: int,
        initial_capacity: int,
        is_overflow: bool,
        options: Optional[StaminaModelOptions] = StaminaModelOptions(),
    ):
        self.name = name
        self.recover_interval_minutes = recover_interval_minutes
        self.recover_value = recover_value
        self.initial_capacity = initial_capacity
        self.is_overflow = is_overflow
        self.metadata = options.metadata if options.metadata else None
        self.max_capacity = options.max_capacity if options.max_capacity else None
        self.max_stamina_table = options.max_stamina_table if options.max_stamina_table else None
        self.recover_interval_table = options.recover_interval_table if options.recover_interval_table else None
        self.recover_value_table = options.recover_value_table if options.recover_value_table else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.recover_interval_minutes is not None:
            properties["recoverIntervalMinutes"] = self.recover_interval_minutes
        if self.recover_value is not None:
            properties["recoverValue"] = self.recover_value
        if self.initial_capacity is not None:
            properties["initialCapacity"] = self.initial_capacity
        if self.is_overflow is not None:
            properties["isOverflow"] = self.is_overflow
        if self.max_capacity is not None:
            properties["maxCapacity"] = self.max_capacity
        if self.max_stamina_table is not None:
            properties["maxStaminaTable"] = self.max_stamina_table.properties(
            )
        if self.recover_interval_table is not None:
            properties["recoverIntervalTable"] = self.recover_interval_table.properties(
            )
        if self.recover_value_table is not None:
            properties["recoverValueTable"] = self.recover_value_table.properties(
            )

        return properties
